               job  Load a deck onto tape
               ctl  6611
     *
     * Write a loader routine on the beginning of unit 3, then write
     * an object deck, modified so that the loader can load it.
     *
     * An end-of-file mark is written after the program.
     *
     * The tape is not rewound before or after copying the program.
     *
     * This program and the loader reader loader use SBR, which is
     * always accompanied by indexing, so there is no compunction
     * against using indexing.  It will run on any size machine except
     * 1.4k.  It doesn't need high-low-equal compare or sense switches.
     *
     * There are two ways to load programs onto tape.
     *
     * The first method is to press start when this program stops with
     * IAR 604 and AAR 611, in which case this program writes a record
     * that the loader reader uses to reset word marks for loading
     * before each overlay.
     *
     * The second method is to press start reset and start, in which
     * case this program writes the loader reader loader, without a
     * clear routine, and the loader reader, before each overlay.
     *
     * In either case, when the loaded program is run from tape, the
     * loader automatically determines core size and loads a reader at
     * x919-x999, where x is core size / 1000 - 1.  The loader reader
     * puts a branch to itself in 81-84.
     *
     * There are two ways to load overlays, depending upon how the tape
     * is written.  A program should have ,,r 40,, at 81-84 and a WM at
     * 85.  This will be preserved if the program is loaded from cards.
     *
     * If the tape is written using the first method, the program must
     * not use x919-x999.  It can load overlays by branching to 81.  If
     * word marks in the read area have been changed the read area
     * should be cleared and a WM set at 40.  This method works to load
     * overlays from cards as well.
     *
     * If the tape is written using the second method, the program
     * should clear 1-80 if there might be a GMWM there, read the next
     * tape record into 1 with word marks, and branch to 1 to load the
     * loader reader loader, which clears x900-x999 and uses x1.  The
     * loader reader loader runs entirely in 1-80.  A program can
     * determine whether to do this or to branch to 81 to load an
     * overlay from cards using
     *
     * bce  81,81,1
     * rtw  1,1
     * b    1
     * Word mark or blank here.
     *
     * This is only 64 characters shorter than the first method.  It
     * also works if the tape was written using the first method,
     * assuming x919-x999 are not damaged.
     *
     * Halts during writing the tape
     *   IAR   AAR   BAR
     *   604   611   ---  start to write loader only at the
     *                    beginning of the tape, start reset and
     *                    start to write it before every overlay.
     *   794   777   777  program successfully loaded to tape.
     *   830   666   666  error while writing program to tape.
     *
     * Halts during loading the program
     *   IAR   AAR   BAR
     *   015   08x   08x  error reading loader reader loader
     *                    record x from tape.  Zero for overlay.
     *   033   ---   067  error reading loader reader from tape.
     *  x927  x923   ---  error loading program record, where x
     *                    is core size / 1000 - 1.
     *
     out       equ  %u3                Output tape number
     x1        equ  089
     *
               org  600
     *
     * Start if loader reader will not be clobbered by the program,
     * in which case an ersatz overlay reloader will be written after
     * each EX record.
     *
     * Start reset and start to write the loader reader and its
     * loader after each EX record.
     *
     start     h    writem
               sbr  bovly&3,write2     setup to reload loader for ovly
     *
     * Write the loader reader loader first two records
     *
     writem    wtw  out,first
               ber  wterr
     write2    wtw  out,second
               ber  wterr     
     *
     * Write the loader reader
     *
               wtw  out,bread
               ber  wterr
     *
     * Write the relocator and cleanup records
     *
               wtw  out,third
               ber  wterr
               wtw  out,fourth
               ber  wterr
     *
     * Copy the deck to tape.  Skip clear core cards.  Replace
     * overlay reloader cards.
     *
     cards     blc  done
               r
               bce  lcard,40,l         q. load card
               bce  wcard,40,n         q. ex card
               c    46,@080@           q. end card
               bu   cards              no
     wcard     lca  gmwm,80
               wt   out,1
               ber  wterr
     bovly     bce  *&5,68,b           EX card
               b    cards
               wt   out,ovlay          write a new overlay reloader
               ber  wterr
               b    cards
     lcard     lca  link&8,76          Insert branch to branch to read
               lca
               b    wcard
     link      sw   85
               bin  81,
     *
     * All done
     *
     done      wtm  out
     hdone     nop  777
               h
               b    hdone
     *
     * Tape write error routine
     *
     wterr     sbr  x1
               sbr  x1,16000-13        Back up to WT instruction
               bsp  out
               a    *-6,werc
               bce  wterrx,werc-1,0
               nop  666
               h
               skp  out
               za   werc
     wterrx    b    0&x1
     werc      dcw  00
     *
     * Ersatz overlay reloader card.  This works either if the program
     * branches to 81 to read the record using the loader reader, or
     * loads the record at 1 with word marks and branches to 1.
     *
               org  901
     ovlay     ber  bad-900
               b    setwms-900
     bad       nop  080
               h
               bsp  1
               rtw  1,1
               b    1
               dc   #4
     setwms    cs   39
               sw   47,54
               sw   61,68
               sw   72,76
               sw   85,1
               b    81
               dc   #8
     gmwm      dcw  @}@
               ltorg*
     *
               job  Loader for I/O routine for loading card images
     *
     * Clear core.  The loader reader loader records are all loaded
     * into 1-80, but we can't assemble them there.
     *
               org  101
     first     ber  bad1-100           This record is in error
               b    sw100-100
     bad1      nop  081                This record is in error.
               h                       Halt with AAR and BAR 81
               bsp  1
     rtw1a     rtw  1,1
               b    1
     sw100     sw   100
     clear     cs   19                 clear core from
               sbr  clear&3-100          top down
               bw   clear-100,100          to 100
               sw   1
               lca  19,99              clear 81-99
               cw   81                   and the WM left by clearing
               b    rtw1a-100
               dcw  #9
               dcw  @}@
     *
     * Compute where to load the loader reader and load it
     *
               org  201
     second    ber  bad2-200           This record is in error
               b    begin2-200
     bad2      nop  082                This record is in error.
               h                       Halt with AAR and BAR 82
               bsp  1
     rtw1b     rtw  1,1
               b    1
     badtop    h                       Loader I/O routine record
               bsp  1                    is in error
     begin2    cs   0                  clear top 100 of core to
               cs                        make sure there is no gmwm
               sbr  x1                 x1 becomes 0, 4k, 8k or 12k
               sbr  x1,12101&x1          for core 4k, 8k, 12k or 16k
               rtw  1,bread&x1         Read the loader read routine
               ber  badtop-200
               mz   *-206,tnoiz&7&x1   A-zone bit for noise testing
               b    rtw1b-200
               dcw  #2
               dcw  @}@
     *
     * Relocate the addresses in the loader reader according to where
     * it was loaded.
     *
               org  301
     third     ber  bad3-300           This record is in error
               b    reloc3-300
     bad3      nop  083                This record is in error.
               h                       Halt with AAR and BAR 83
               bsp  1
     rtw1c     rtw  1,1
               b    1
     reloc3    sbr  bread&3&x1,read&x1
               sbr  tnoiz&3&x1
               sbr  ehalt&3&x1,ehalt&x1
               sbr  l4&3&x1
               sbr  l2&3&x1,errct&x1
               sbr  rderr&6&x1
               lca  k00-300,errct&x1   Initialize error counter
               b    rtw1c-300
     k00       dcw  00
               dcw  #2
               dcw  @}@
     *
     * Relocate more of the addresses in the loader reader.
     *
               org  401
     fourth    ber  bad4-400           This record is in error
               b    reloc4-400
     bad4      nop  084                This record is in error.
               h                       Halt with AAR and BAR 84
               bsp  1
               rtw  1,1
               bin  1,
     reloc4    sbr  l4&6&x1,errct-1&x1
               sbr  read&3&x1,tnoiz&7&x1
               sbr  l9&3&x1,rderr&x1
               sbr  rderr&3&x1,rderr&x1
               sbr  l13&3&x1,bread&3&x1
               cs   39                 Clear WMs in 1-39
               b    read&x1            Enter loader reader
               dcw  #1
               dcw  @}@
               job  High core I/O routine for loading card images
     *
     * The loader reader has a better error handler, and a noise
     * record test.
     *
     * It always clears 1-39 in case 1 was a GM on a previous read.
     * Therefore, unlike the one that Sort 7 puts on tapes, this one
     * doesn't need records to be fiddled if they have GM in 1.
     *
     * It leaves ,,B  READ,, in 81-84, which programs can use to
     * load overlays without knowing where the loader reader is
     * loaded.
     *
               org  3919
     bread     b    read
     ehalt     h    ehalt              Start reset to try again
     l2        za   errct              Clear error counter
     rderr     a    rderr,errct        Bump error counter
     l4        bce  ehalt,errct-1,9    Q. 90 errors altogether
               bsp  1
     read      mcw  tnoiz&7,40         A-zone bit for noise testing
               cs                      Clear any GMARKs in 1-39
               rt   1,1                Read a loader record
     l9        ber  rderr              Q. tape read error
     tnoiz     bce  read,40,           Test for A-zone bit
     l13       lca  bread&3,84         Set link to here into 81-84
               sw   1,40               WM in 40 for overlay reloader
               b    40                 Run the loader in the record
               dcw  @}@
     errct     dc   #1
               end  start
